'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var dom = require('solid-js/dom');
var history = require('history');
var solidJs = require('solid-js');
var pathToRegexp = require('path-to-regexp');

const HistoryContext = solidJs.createContext();
function ContextProvider(props) {
  return dom.createComponent(HistoryContext.Provider, {
    value: () => props.history,
    children: () => props.children
  }, ["value", "children"]);
}

function ServerContextProvider(props) {
  return dom.createComponent(ContextProvider, {
    history: () => history.createMemoryHistory(props.options),
    children: () => props.children
  }, ["history", "children"]);
}

function getArrayOf(array) {
  if (Array.isArray(array)) {
    return array;
  }

  if (!!array) {
    return [array];
  }

  return [];
}

const RouteContext = solidJs.createContext();

function createRouteHandler() {
  const history = solidJs.useContext(HistoryContext);
  const [location, setLocation] = solidJs.createSignal(history.location);
  const locationHandler = history.listen(location => {
    setLocation(location);
  });
  solidJs.onCleanup(() => locationHandler());
  return route => {
    const matcher = pathToRegexp.match(route.path || '', route.options);
    return () => {
      return matcher(location().pathname);
    };
  };
}

function Router(props) {
  const createMatcher = createRouteHandler();
  const routes = getArrayOf(props.children);
  const matchers = routes.map(createMatcher);
  const useFallback = 'fallback' in props;
  const evalConditions = solidJs.createMemo(() => matchers.reduce((result, matcher, index) => {
    if (result.index === -1) {
      const match = matcher();

      if (match) {
        return {
          index,
          params: match.params
        };
      }
    }

    return result;
  }, {
    index: -1,
    params: {}
  }));
  return solidJs.awaitSuspense(solidJs.createMemo(() => {
    const {
      index,
      params
    } = evalConditions();
    return solidJs.sample(() => dom.createComponent(RouteContext.Provider, {
      value: params,
      children: () => index < 0 ? useFallback && props.fallback : routes[index].children
    }, ["children"]));
  }));
}

function Route(props) {
  return props;
}

const _tmpl$ = dom.template(`<a></a>`);
function Link(props) {
  const history = solidJs.useContext(HistoryContext);

  return function () {
    //@ts-ignore
    const _el$ = dom.getNextElement(_tmpl$, true);

    dom.spread(_el$, props, false, false);

    return _el$;
  }();
}

function useHistory() {
  return solidJs.useContext(HistoryContext);
}

function useRoute() {
  return solidJs.useContext(RouteContext);
}

exports.ContextProvider = ServerContextProvider;
exports.Link = Link;
exports.Route = Route;
exports.Router = Router;
exports.useHistory = useHistory;
exports.useRoute = useRoute;
