import { createComponent, getNextElement, spread, template } from 'solid-js/dom';
import { createMemoryHistory } from 'history';
import { createContext, createMemo, awaitSuspense, sample, useContext, createSignal, onCleanup } from 'solid-js';
import { match } from 'path-to-regexp';

const HistoryContext = createContext();
function ContextProvider(props) {
  return createComponent(HistoryContext.Provider, {
    value: () => props.history,
    children: () => props.children
  }, ["value", "children"]);
}

function ServerContextProvider(props) {
  return createComponent(ContextProvider, {
    history: () => createMemoryHistory(props.options),
    children: () => props.children
  }, ["history", "children"]);
}

function getArrayOf(array) {
  if (Array.isArray(array)) {
    return array;
  }

  if (!!array) {
    return [array];
  }

  return [];
}

const RouteContext = createContext();

function createRouteHandler() {
  const history = useContext(HistoryContext);
  const [location, setLocation] = createSignal(history.location);
  const locationHandler = history.listen(location => {
    setLocation(location);
  });
  onCleanup(() => locationHandler());
  return route => {
    const matcher = match(route.path || '', route.options);
    return () => {
      return matcher(location().pathname);
    };
  };
}

function Router(props) {
  const createMatcher = createRouteHandler();
  const routes = getArrayOf(props.children);
  const matchers = routes.map(createMatcher);
  const useFallback = 'fallback' in props;
  const evalConditions = createMemo(() => matchers.reduce((result, matcher, index) => {
    if (result.index === -1) {
      const match = matcher();

      if (match) {
        return {
          index,
          params: match.params
        };
      }
    }

    return result;
  }, {
    index: -1,
    params: {}
  }));
  return awaitSuspense(createMemo(() => {
    const {
      index,
      params
    } = evalConditions();
    return sample(() => createComponent(RouteContext.Provider, {
      value: params,
      children: () => index < 0 ? useFallback && props.fallback : routes[index].children
    }, ["children"]));
  }));
}

function Route(props) {
  return props;
}

const _tmpl$ = template(`<a></a>`);
function Link(props) {
  const history = useContext(HistoryContext);

  return function () {
    //@ts-ignore
    const _el$ = getNextElement(_tmpl$, true);

    spread(_el$, props, false, false);

    return _el$;
  }();
}

function useHistory() {
  return useContext(HistoryContext);
}

function useRoute() {
  return useContext(RouteContext);
}

export { ServerContextProvider as ContextProvider, Link, Route, Router, useHistory, useRoute };
